// band screen saver 
// by douglas irving repetto
// based on code from be inc.
//
// selects one row or column on screen and 
// fills screen with it

#include <add-ons/screen_saver/ScreenSaver.h> 
#include <interface/StringView.h> 
#include <Bitmap.h>
#include <BeBuild.h> 
#include <stdlib.h>

extern "C" { 
_EXPORT BScreenSaver *instantiate_screen_saver( BMessage *msg, image_id id ); 
} 

class MinimalSaver : public BScreenSaver 
{ 
public: 
    MinimalSaver( BMessage *archive, image_id image ); 
    virtual ~MinimalSaver( void ); 
    
    virtual void StartConfig( BView *view ); 
    virtual status_t StartSaver( BView *view, bool preview ); 
	virtual void StopSaver();
    virtual void Draw( BView *view, int32 frame ); 

	bool		doingPreview;
	BBitmap* mScreenBitmap;
}; 

BScreenSaver *instantiate_screen_saver( BMessage *msg, image_id id ) 
{ 
    return new MinimalSaver( msg, id ); 
} 

MinimalSaver::MinimalSaver( BMessage *archive, image_id image ) 
    : BScreenSaver( archive, image ) 
{     
	mScreenBitmap = 0;
    SetTickSize(0);
} 

MinimalSaver::~MinimalSaver( void ) 
{ 
	delete mScreenBitmap;
    // Nothing else to do here... 
} 

void MinimalSaver::StartConfig( BView *view ) 
{ 
    view->AddChild( new BStringView( BRect( 10, 10, 200, 35 ), B_EMPTY_STRING, 
		"band screensaver, by d.i.r." ) );
} 

status_t MinimalSaver::StartSaver( BView *view, bool preview ) 
{ 
	BRect screenFrame;
	
    if (preview)
    	doingPreview = true;
    else
    	doingPreview = false;

	srandom(system_time());
	
	BScreen screen(B_MAIN_SCREEN_ID);
	screenFrame = screen.Frame();
	screenFrame.PrintToStream();
	
	/* a 32-bit screen bitmap is obviously large and perhaps
	** wasteful when the actual screen bit depth is lower, but
	** it makes grabbing the pixel values in GetPixel easy
	*/
	//we'll save the original screen in here
	mScreenBitmap = new BBitmap(screenFrame,B_RGB32,false);
	
	if (mScreenBitmap->IsValid()) {
		// perhaps BScreen::GetBitmap is the ticket now? Investigate...
		//_get_screen_bitmap_(mScreenBitmap,screenFrame,false);
		screen.ReadBitmap(mScreenBitmap);
	}
	else {
		return (B_ERROR);
	}

    return B_OK; 
} 

void MinimalSaver::StopSaver()
{
}

void MinimalSaver::Draw( BView *view, int32 frame ) 
{ 
	BRect rect = view->Bounds();
	static int32 which = 0;
	static BRect staticRect(0.0, which, rect.right, which + 1);
	static BRect currRect(0.0, which, rect.right, which + 1);
	static int32 previewCount = 0;
	static int increment = 1;
	static int32 orientation = random()%4;
	static bool newOrientation = false;
	
	//if we're in the preview, only draw every 20 frames...
	if (!doingPreview || previewCount % 10 == 0) {
		//we've completed a wipe, so pick a new direction
		if (newOrientation) {
			orientation = random()%4;
			if (orientation == 0 || orientation == 1) {
				int32 row = random()%((int32)rect.Height() - 1);
				staticRect.top = row;
				staticRect.bottom = row + 1;
				staticRect.left = rect.left;
				staticRect.right = rect.right;
				increment = (orientation == 0) ? 1:-1;
				which = (orientation == 0) ? 0:rect.Height();
				newOrientation = false;
			}
			else {
				int32 column = random()%((int32)rect.Width() - 1);
				staticRect.left = column;
				staticRect.right = column + 1;
				staticRect.top = rect.top;
				staticRect.bottom = rect.bottom;
				increment = (orientation == 2) ? 1:-1;
				which = (orientation == 2) ? 0:rect.Width();
				newOrientation = false;
			}
		}
		
		//top to bottom or bottom to top
		if (orientation == 0 || orientation == 1) {
			currRect = BRect(0.0, which, rect.right, which + 1);		
			which += increment;
			if (which >= (int32)rect.Height() || which <= 0)
				newOrientation = true;			
		}		
		//left to right or right to left	
		else {
			currRect = BRect(which, 0.0, which + 1, rect.bottom);
			which += increment;
			if (which >= (int32)rect.Width() || which <= 0)
				newOrientation = true;			
		}		

		if (view->Window()->Lock()) {
			view->DrawBitmap(mScreenBitmap, staticRect, currRect);
			view->Window()->Unlock();
		}
	}
	previewCount++;
}