// nausea screen saver 
// by douglas irving repetto
// based on code from be inc.
//
// zips around the screen making you sick...

#include <add-ons/screen_saver/ScreenSaver.h> 
#include <interface/StringView.h> 
#include <BeBuild.h> 
#include <Bitmap.h>
#include <File.h>
#include <stdlib.h>
#include <stdio.h>

extern "C" { 
_EXPORT BScreenSaver *instantiate_screen_saver( BMessage *msg, image_id id ); 
} 

class MinimalSaver : public BScreenSaver 
{ 
public: 
    MinimalSaver( BMessage *archive, image_id image ); 
    virtual ~MinimalSaver( void ); 
    
    virtual void StartConfig( BView *view ); 
    virtual status_t StartSaver( BView *view, bool preview ); 
    virtual void Draw( BView *view, int32 frame ); 
    
	bool		doingPreview;
	BRect screenFrame;
	BBitmap* mScreenBitmap;
	
}; 

BScreenSaver *instantiate_screen_saver( BMessage *msg, image_id id ) 
{ 
    return new MinimalSaver( msg, id ); 
} 

MinimalSaver::MinimalSaver( BMessage *archive, image_id image ) 
    : BScreenSaver( archive, image ) 
{     
	mScreenBitmap = 0;
    SetTickSize(10000);
} 

MinimalSaver::~MinimalSaver( void ) 
{ 
	delete mScreenBitmap;
    // Nothing else to do here... 
} 

void MinimalSaver::StartConfig( BView *view ) 
{ 
    view->AddChild( new BStringView( BRect( 10, 10, 200, 35 ), 
                                     B_EMPTY_STRING, 
                                     "nausea screensaver, by d.i.r." ) ); 
} 

status_t MinimalSaver::StartSaver( BView *view, bool preview ) 
{ 
	printf("StartSaver()\n");

    if (preview)
    	doingPreview = true;
    else
    	doingPreview = false;

	srandom(system_time());
	
	BScreen screen(B_MAIN_SCREEN_ID);
	screenFrame = screen.Frame();
	screenFrame.PrintToStream();
	
	/* a 32-bit screen bitmap is obviously large and perhaps
	** wasteful when the actual screen bit depth is lower, but
	** it makes grabbing the pixel values in GetPixel easy
	*/
	//we'll save the original screen in here
	mScreenBitmap = new BBitmap(screenFrame,B_RGB32,false);
	
	if (mScreenBitmap->IsValid()) {
		screen.ReadBitmap(mScreenBitmap);
	}
	else {
		return (B_ERROR);
	}

    return B_OK; 
} 

void MinimalSaver::Draw( BView *view, int32 frame ) 
{ 
	BRect viewFrame = view->Bounds();
	static BRect sourceRect(-1.0,-1.0,-1.0,-1.0);
	static int32 previewCount = 0;
	static int lIncrement = 0;
	static int rIncrement = 0;
	static int tIncrement = 0;
	static int bIncrement = 0;
	static bool firstTime = true;
	

	//if we're in the preview, only draw every 8 frames...
	if (!doingPreview || previewCount % 8 == 0) {
		if (firstTime) {
			//get initial rect
			sourceRect.left = random()%(int32)viewFrame.Width();
			sourceRect.right = sourceRect.left + 
				random()%(int32)viewFrame.Width();	
			sourceRect.top = random()%(int32)viewFrame.Height();
			sourceRect.bottom = sourceRect.top + 
				random()%(int32)viewFrame.Height();
			//see if we got a valid rect. if not, try again until we do
			while(!viewFrame.Contains(sourceRect) || sourceRect.Width() < 5 || 
					sourceRect.Height() < 5) {
				//get new sides
				sourceRect.left = random()%(int32)viewFrame.Width();
				sourceRect.right = sourceRect.left + 
					random()%(int32)viewFrame.Width();	
				sourceRect.top = random()%(int32)viewFrame.Height();
				sourceRect.bottom = sourceRect.top + 
					random()%(int32)viewFrame.Height();
			}
			lIncrement = random()%31 - 15;
			rIncrement = random()%31 - 15;
			tIncrement = random()%31 - 15;
			bIncrement = random()%31 - 15;
			firstTime = false;
		}
		else {
			//resize sourceRect
			BRect temp = sourceRect;	
			temp.left += lIncrement;
			temp.right += rIncrement;
			temp.top += tIncrement;
			temp.bottom += bIncrement;
			//if Width() or Height() is too small, and we're at the edge
			//of the screen, screen_saver crashes... weird?!?
			while (!viewFrame.Contains(temp) || temp.Width() < 5 || temp.Height() < 5) {
				temp = sourceRect;	
				lIncrement = random()%31 - 15;
				rIncrement = random()%31 - 15;
				tIncrement = random()%31 - 15;
				bIncrement = random()%31 - 15;
				temp.left += lIncrement;
				temp.right += rIncrement;
				temp.top += tIncrement;
				temp.bottom += bIncrement;
			}
			sourceRect = temp;
		}
		if (view->Window()->Lock()) {
			//blow up sourceRect to fit into destRect
			view->DrawBitmap(mScreenBitmap, sourceRect, viewFrame);
			//use this if you want to see the original source in a little rect
			//view->DrawBitmap(mScreenBitmap, sourceRect, sourceRect);
			//stroke outline
			//view->SetHighColor(0,0,255);
			//view->StrokeRect(sourceRect);
			view->Window()->Unlock();
		}
	}
	previewCount++;
}